@@ -63,6 +63,27 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
 logging.getLogger().setLevel(logging.DEBUG)
 
+
+def _get_last_aider_reply(directory_path: str) -> Optional[str]:
+    """
+    Read the Aider chat history, extract the last session, clean SEARCH/REPLACE noise,
+    and return the snippet or None if unavailable.
+    """
+    try:
+        history_path = Path(directory_path) / ".aider.chat.history.md"
+        if not history_path.exists():
+            return None
+        history_content = history_path.read_text(encoding="utf-8", errors="ignore")
+        anchor = "# aider chat started at"
+        last_anchor_pos = history_content.rfind(anchor)
+        snippet = history_content[last_anchor_pos:] if last_anchor_pos != -1 else history_content
+        # Remove SEARCH/REPLACE noise blocks
+        snippet = re.sub(r"<<<<<<< SEARCH.*?>>>>>>> REPLACE", "", snippet, flags=re.DOTALL)
+        return snippet.strip() or None
+    except Exception as e:
+        logger.debug(f"Failed to get Aider chat history: {e}")
+        return None
+
 # === AI_HINT helper builders (keep terse, agent-friendly) ===
 
 def ai_hint_git_apply_diff_error(stderr: str | None, affected_file_path: str | None) -> str:
@@ -87,13 +108,6 @@ def ai_hint_read_file_error(file_path: str, repo_working_dir: str, e: Exception)
         "Ensure the file exists and is readable, and that you passed an absolute repo_path."
     )
 
-def ai_hint_sed_error(e: Exception) -> str:
-    return (
-        "UNEXPECTED_ERROR: An unexpected error occurred during sed-based search and replace: "
-        f"{e}. If your pattern contains special characters, prefer simpler patterns or rely on Python fallback. "
-        "Ensure the file exists and is writable, and pass an absolute repo_path."
-    )
-
 def ai_hint_write_error(repo_path: str, file_path: str, e: Exception) -> str:
     return (
         "UNEXPECTED_ERROR: Failed to write to file "
@@ -335,18 +349,16 @@ class GitStatus(BaseModel):
     """
     repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
 
-class GitDiffAll(BaseModel):
-    """
-    Represents the input schema for the `git_diff_all` tool.
-    """
-    repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
 
 class GitDiff(BaseModel):
     """
     Represents the input schema for the `git_diff` tool.
     """
     repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
-    target: str = Field(description="The target (e.g., branch name, commit hash, tag) to diff against. For example, 'main', 'HEAD~1', or a full commit SHA.")
+    target: Optional[str] = Field(
+        None,
+        description="Optional. If omitted, behaves like `git diff` (worktree vs index). Pass 'HEAD' or another ref to compare against a commit or branch."
+    )
 
 class GitCommit(BaseModel):
     """
@@ -411,31 +423,17 @@ class GitReadFile(BaseModel):
     repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
     file_path: str = Field(description="The path to the file to read, relative to the repository's working directory.")
 
-class SearchAndReplace(BaseModel):
-    """
-    Represents the input schema for the `search_and_replace` tool.
-    """
-    repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
-    file_path: str = Field(description="The path to the file to modify, relative to the repository's working directory.")
-    search_string: str = Field(description="The string or regex pattern to search for within the file.")
-    replace_string: str = Field(description="The string to replace all matches of the search string with.")
-    ignore_case: bool = Field(False, description="If true, the search will be case-insensitive. Defaults to false.")
-    start_line: Optional[int] = Field(
-        None,
-        description="Optional. The 1-based starting line number for the search and replace operation (inclusive). If not provided, search starts from the beginning of the file."
-    )
-    end_line: Optional[int] = Field(
-        None,
-        description="Optional. The 1-based ending line number for the search and replace operation (inclusive). If not provided, search continues to the end of the file."
-    )
-
 class WriteToFile(BaseModel):
     """
     Represents the input schema for the `write_to_file` tool.
     """
     repo_path: str = Field(description="The absolute path to the Git repository's working directory.")
-    file_path: str = Field(description="The path to the file to write to, relative to the repository's working directory. The file will be created if it doesn't exist, or overwritten if it does.")
+    file_path: str = Field(description="The path to the file to write to, relative to the repository's working directory. The file will be created if it doesn't exist. Existing files are never overwritten unless overwrite=true is explicitly provided.")
     content: str = Field(description="The string content to write to the specified file.")
+    overwrite: bool = Field(
+        default=False,
+        description="If true, allows overwriting an existing file. Defaults to false."
+    )
 
 class ExecuteCommand(BaseModel):
     """
@@ -495,7 +493,6 @@ class GitTools(str, Enum):
     An enumeration of all available Git and related tools.
     """
     STATUS = "git_status"
-    DIFF_ALL = "git_diff_all"
     DIFF = "git_diff"
     STAGE_AND_COMMIT = "git_stage_and_commit"
     RESET = "git_reset"
@@ -505,7 +502,6 @@ class GitTools(str, Enum):
     SHOW = "git_show"
     APPLY_DIFF = "git_apply_diff"
     READ_FILE = "git_read_file"
-    SEARCH_AND_REPLACE = "search_and_replace"
     WRITE_TO_FILE = "write_to_file"
     EXECUTE_COMMAND = "execute_command"
     AI_EDIT = "ai_edit"
@@ -523,30 +519,21 @@ def git_status(repo: git.Repo) -> str:
     """
     return repo.git.status()
 
-def git_diff_all(repo: git.Repo) -> str:
+
+def git_diff(repo: git.Repo, target: Optional[str] = None) -> str:
     """
-    Shows all changes in the working directory (staged and unstaged, compared to HEAD).
+    Shows differences in the working directory. If target is None, shows worktree vs index.
+    If target is provided, shows differences against that target.
 
     Args:
         repo: The Git repository object.
+        target: Optional. The target (branch, commit hash, etc.) to diff against.
+                If None, behaves like `git diff` (worktree vs index).
 
     Returns:
-        A string representing the output of `git diff HEAD`.
+        A string representing the output of `git diff` or `git diff <target>`.
     """
-    return repo.git.diff("HEAD")
-
-def git_diff(repo: git.Repo, target: str) -> str:
-    """
-    Shows differences between branches or commits.
-
-    Args:
-        repo: The Git repository object.
-        target: The target (branch, commit hash, etc.) to diff against.
-
-    Returns:
-        A string representing the output of `git diff <target>`.
-    """
-    return repo.git.diff(target)
+    return repo.git.diff() if target is None else repo.git.diff(target)
 
 def git_stage_and_commit(repo: git.Repo, message: str, files: Optional[List[str]] = None) -> str:
     """
@@ -803,195 +790,17 @@ async def _run_tsc_if_applicable(repo_path: str, file_path: str) -> str:
         return f"\n\nTSC Output for {file_path}:\n{tsc_output}"
     return ""
 
-async def _search_and_replace_python_logic(
-    repo_path: str,
-    search_string: str,
-    replace_string: str,
-    file_path: str,
-    ignore_case: bool,
-    start_line: Optional[int],
-    end_line: Optional[int]
-) -> str:
+
+async def write_to_file_content(repo_path: str, file_path: str, content: str, overwrite: bool = False) -> str:
     """
-    Performs search and replace in a file using Python's re module.
-    Attempts literal search first, then falls back to regex search if no changes are made.
-
-    Args:
-        repo_path: The path to the repository's working directory.
-        search_string: The string or regex pattern to search for.
-        replace_string: The string to replace matches with.
-        file_path: The path to the file to modify.
-        ignore_case: If True, the search is case-insensitive.
-        start_line: Optional. The 1-based starting line number for the search.
-        end_line: Optional. The 1-based ending line number for the search.
-
-    Returns:
-        A string indicating the result of the operation, including diff and TSC output,
-        or an error message.
-    """
-    try:
-        full_file_path = Path(repo_path) / file_path
-        with open(full_file_path, 'r') as f:
-            lines = f.readlines()
-
-        flags = 0
-        if ignore_case:
-            flags |= re.IGNORECASE
-
-        literal_search_string = re.escape(search_string)
-        logging.info(f"Attempting literal search with: {literal_search_string}")
-
-        modified_lines_literal = []
-        changes_made_literal = 0
-
-        for i, line in enumerate(lines):
-            line_num = i + 1
-            if (start_line is None or line_num >= start_line) and \
-               (end_line is None or line_num <= end_line):
-                new_line, num_subs = re.subn(literal_search_string, replace_string, line, flags=flags)
-                
-                if new_line != line:
-                    changes_made_literal += num_subs
-                    modified_lines_literal.append(new_line)
-                else:
-                    modified_lines_literal.append(line)
-            else:
-                modified_lines_literal.append(line)
-
-        if changes_made_literal > 0:
-            original_content = "".join(lines)
-            with open(full_file_path, 'w') as f:
-                f.writelines(modified_lines_literal)
-            
-            result_message = f"Successfully replaced '{search_string}' with '{replace_string}' in {file_path} using literal search. Total changes: {changes_made_literal}."
-            result_message += await _generate_diff_output(original_content, "".join(modified_lines_literal), file_path)
-            result_message += await _run_tsc_if_applicable(repo_path, file_path)
-            return result_message
-        else:
-            logging.info(f"Literal search failed. Attempting regex search with: {search_string}")
-            modified_lines_regex = []
-            changes_made_regex = 0
-            
-            for i, line in enumerate(lines):
-                line_num = i + 1
-                if (start_line is None or line_num >= start_line) and \
-                   (end_line is None or line_num <= end_line):
-                    new_line, num_subs = re.subn(search_string, replace_string, line, flags=flags)
-                    
-                    if new_line != line:
-                        changes_made_regex += num_subs
-                        modified_lines_regex.append(new_line)
-                    else:
-                        modified_lines_regex.append(line)
-                else:
-                    modified_lines_regex.append(line)
-
-            if changes_made_regex > 0:
-                original_content = "".join(lines)
-                with open(full_file_path, 'w') as f:
-                    f.writelines(modified_lines_regex)
-                
-                result_message = f"Successfully replaced '{search_string}' with '{replace_string}' in {file_path} using regex search. Total changes: {changes_made_regex}."
-                result_message += await _generate_diff_output(original_content, "".join(modified_lines_regex), file_path)
-                result_message += await _run_tsc_if_applicable(repo_path, file_path)
-                return result_message
-            else:
-                return f"No changes made. '{search_string}' not found in {file_path} within the specified range using either literal or regex search."
-
-    except FileNotFoundError:
-        return f"Error: File not found at {full_file_path}"
-    except re.error as e:
-        return f"Error: Invalid regex pattern '{search_string}': {e}"
-    except Exception as e:
-        return f"UNEXPECTED_ERROR: An unexpected error occurred during search and replace: {e}. AI_HINT: Check your search/replace patterns and review server logs for more details."
-
-async def search_and_replace_in_file(
-    repo_path: str,
-    search_string: str,
-    replace_string: str,
-    file_path: str,
-    ignore_case: bool,
-    start_line: Optional[int],
-    end_line: Optional[int]
-) -> str:
-    """
-    Searches for a string or regex pattern in a file and replaces it with another string.
-    Attempts to use `sed` for efficiency, falling back to Python logic if `sed` fails or makes no changes.
-
-    Args:
-        repo_path: The path to the repository's working directory.
-        search_string: The string or regex pattern to search for.
-        replace_string: The string to replace matches with.
-        file_path: The path to the file to modify.
-        ignore_case: If True, the search is case-insensitive.
-        start_line: Optional. The 1-based starting line number for the search.
-        end_line: Optional. The 1-based ending line number for the search.
-
-    Returns:
-        A string indicating the result of the operation, including diff and TSC output,
-        or an error message.
-    """
-    full_file_path = Path(repo_path) / file_path
-
-    sed_command_parts = ["sed", "-i"]
-
-    # Escape sed special characters in search and replacement strings
-    sed_pattern = search_string.replace('#', r'\#')
-    sed_replacement = replace_string.replace('#', r'\#').replace('&', r'\&').replace('\\', r'\\\\')
-
-    sed_flags = "g"
-    if ignore_case:
-        sed_flags += "i"
-
-    sed_sub_command = f"s#{sed_pattern}#{sed_replacement}#{sed_flags}"
-
-    if start_line is not None and end_line is not None:
-        sed_sub_command = f"{start_line},{end_line}{sed_sub_command}"
-    elif start_line is not None:
-        sed_sub_command = f"{start_line},${sed_sub_command}"
-    elif end_line is not None:
-        sed_sub_command = f"1,{end_line}{sed_sub_command}"
-
-    sed_full_command = f"{' '.join(sed_command_parts)} '{sed_sub_command}' {shlex.quote(str(full_file_path))}"
-
-    try:
-        with open(full_file_path, 'r') as f:
-            original_content = f.read()
-
-        sed_result = await execute_custom_command(repo_path, sed_full_command)
-        logging.info(f"Sed command result: {sed_result}")
-
-        if "Command failed with exit code" in sed_result or "Error executing command" in sed_result:
-            logging.warning(f"Sed command failed: {sed_result}. Falling back to Python logic.")
-            return await _search_and_replace_python_logic(repo_path, search_string, replace_string, file_path, ignore_case, start_line, end_line)
-        
-        with open(full_file_path, 'r') as f:
-            modified_content_sed = f.read()
-
-        if original_content != modified_content_sed:
-            result_message = f"Successfully replaced '{search_string}' with '{replace_string}' in {file_path} using sed."
-            result_message += await _generate_diff_output(original_content, modified_content_sed, file_path)
-            result_message += await _run_tsc_if_applicable(repo_path, file_path)
-            return result_message
-        else:
-            logging.info(f"Sed command executed but made no changes. Falling back to Python logic.")
-            return await _search_and_replace_python_logic(repo_path, search_string, replace_string, file_path, ignore_case, start_line, end_line)
-
-    except FileNotFoundError:
-        return f"Error: File not found at {full_file_path}"
-    except Exception as e:
-        logging.error(f"An unexpected error occurred during sed attempt: {e}. Falling back to Python logic.")
-        return ai_hint_sed_error(e)
-
-async def write_to_file_content(repo_path: str, file_path: str, content: str) -> str:
-    """
-    Writes content to a specified file, creating it if it doesn't exist or overwriting it if it does.
+    Writes content to a specified file, creating it if it doesn't exist.
     Includes a check to ensure the content was written correctly and generates a diff.
 
     Args:
         repo_path: The path to the repository's working directory.
         file_path: The path to the file to write to, relative to the repository.
         content: The string content to write to the file.
+        overwrite: If True, allows overwriting existing files. Defaults to False.
 
     Returns:
         A string indicating the success of the write operation, including diff and TSC output,
@@ -1000,8 +809,12 @@ async def write_to_file_content(repo_path: str, file_path: str, content: str) ->
     try:
         full_file_path = Path(repo_path) / file_path
         
-        original_content = ""
+        # Check if file exists and overwrite protection is enabled
         file_existed = full_file_path.exists()
+        if file_existed and not overwrite:
+            return f"OVERWRITE_PROTECTED: File already exists: {file_path}. Pass overwrite=true to overwrite."
+        
+        original_content = ""
         if file_existed:
             with open(full_file_path, 'r') as f:
                 original_content = f.read()
@@ -1295,12 +1108,24 @@ async def ai_edit_files(
                     else:
                         result_message += "\n\nNo new commit detected or no changes made by Aider."
 
+                    # Append the last Aider reply from chat history
+                    last_reply = _get_last_aider_reply(directory_path)
+                    if last_reply:
+                        result_message += f"\n\nAider last reply (from chat log):\n{last_reply}"
+
                 except git.InvalidGitRepositoryError:
                     result_message += "\n\nCould not access Git repository to get diff after Aider run."
                 except Exception as e:
                     result_message += f"\n\nError generating diff for Aider changes: {e}"
             else:
-                 result_message += f"\nIt's unclear if changes were applied. Please verify the file manually.\nSTDOUT:\n{stdout}"
+                 result_message += (f"\nIt's unclear if changes were applied. Please verify the file manually.\n"
+                                     f"You can also inspect .aider.chat.history.md in the repo root for Aider's chat log.\n"
+                                     f"STDOUT:\n{stdout}")
+            
+            # Append the last Aider reply from chat history
+            last_reply = _get_last_aider_reply(directory_path)
+            if last_reply:
+                result_message += f"\n\nAider last reply (from chat log):\n{last_reply}"
             
             return result_message
 
@@ -1419,14 +1244,9 @@ async def list_tools() -> list[Tool]:
             description="Shows the current status of the Git working tree, including untracked, modified, and staged files.",
             inputSchema=GitStatus.model_json_schema(),
         ),
-        Tool(
-            name=GitTools.DIFF_ALL,
-            description="Shows all changes in the working directory, including both staged and unstaged modifications, compared to the HEAD commit. This provides a comprehensive view of all local changes.",
-            inputSchema=GitDiffAll.model_json_schema(),
-        ),
         Tool(
             name=GitTools.DIFF,
-            description="Shows differences between the current working directory and a specified Git target (e.g., another branch, a specific commit hash, or a tag).",
+            description="Shows differences in the working directory. By default (without target), shows worktree vs index like `git diff`. Pass target='HEAD' for previous 'all changes vs HEAD' behavior.",
             inputSchema=GitDiff.model_json_schema(),
         ),
         Tool(
@@ -1469,14 +1289,9 @@ async def list_tools() -> list[Tool]:
             description="Reads and returns the entire content of a specified file within the Git repository's working directory. The file path must be relative to the repository root.",
             inputSchema=GitReadFile.model_json_schema(),
         ),
-        Tool(
-            name=GitTools.SEARCH_AND_REPLACE,
-            description="Searches for a specified string or regex pattern within a file and replaces all occurrences with a new string. Supports case-insensitive search and line-range restrictions. It attempts to use `sed` for efficiency, falling back to Python logic if `sed` fails or makes no changes.",
-            inputSchema=SearchAndReplace.model_json_schema(),
-        ),
         Tool(
             name=GitTools.WRITE_TO_FILE,
-            description="Writes the provided content to a specified file within the repository. If the file does not exist, it will be created. If it exists, its content will be completely overwritten. Includes a check to ensure content was written correctly and generates a diff.",
+            description="Writes the provided content to a specified file within the repository. If the file does not exist, it will be created. Existing files are never overwritten unless overwrite=true is explicitly provided. Includes a check to ensure content was written correctly and generates a diff.",
             inputSchema=WriteToFile.model_json_schema(),
         ),
         Tool(
@@ -1645,19 +1460,14 @@ async def call_tool(name: str, arguments: dict) -> list[Content]:
                         type="text",
                         text=f"Repository status:\n{status}"
                     )]
-                case GitTools.DIFF_ALL:
-                    repo = git.Repo(repo_path)
-                    diff = git_diff_all(repo)
-                    return [TextContent(
-                        type="text",
-                        text=f"All changes (staged and unstaged):\n{diff}"
-                    )]
                 case GitTools.DIFF:
                     repo = git.Repo(repo_path)
-                    diff = git_diff(repo, arguments["target"])
+                    target = arguments.get("target")
+                    diff = git_diff(repo, target)
+                    diff_header = f"Diff with {target}:" if target else "Diff of unstaged changes (worktree vs index):"
                     return [TextContent(
                         type="text",
-                        text=f"Diff with {arguments['target']}:\n{diff}"
+                        text=f"{diff_header}\n{diff}"
                     )]
                 case GitTools.STAGE_AND_COMMIT:
                     repo = git.Repo(repo_path)
@@ -1719,26 +1529,13 @@ async def call_tool(name: str, arguments: dict) -> list[Content]:
                         type="text",
                         text=result
                     )]
-                case GitTools.SEARCH_AND_REPLACE:
-                    result = await search_and_replace_in_file(
-                        repo_path=str(repo_path),
-                        file_path=arguments["file_path"],
-                        search_string=arguments["search_string"],
-                        replace_string=arguments["replace_string"],
-                        ignore_case=arguments.get("ignore_case", False),
-                        start_line=arguments.get("start_line"),
-                        end_line=arguments.get("end_line")
-                    )
-                    return [TextContent(
-                        type="text",
-                        text=result
-                    )]
                 case GitTools.WRITE_TO_FILE:
                     logging.debug(f"Content input to write_to_file: {arguments['content']}")
                     result = await write_to_file_content(
                         repo_path=str(repo_path),
                         file_path=arguments["file_path"],
-                        content=arguments["content"]
+                        content=arguments["content"],
+                        overwrite=arguments.get("overwrite", False)
                     )
                     logging.debug(f"Content before TextContent: {result}")
                     return [TextContent(
